@@ -7,11 +7,11 @@ from typing import (Callable, Dict, Iterable, List, Optional, Sequence, Tuple,
                     Union)
 
 import numpy as np
+from genbase import Readable
 from instancelib.instances.text import MemoryTextInstance, TextInstance
 from instancelib.typehints import KT, LT
 from nlpaug.base_augmenter import Augmenter
 from text_explainability.decorators import text_instance
-from text_explainability.default import Readable
 from text_explainability.utils import default_detokenizer, default_tokenizer
 
 
@@ -170,7 +170,7 @@ class OneToOnePerturbation(Perturbation):
 
             Replace the character '.' with '!' (character-level replacement):
             >>> from text_explainability import character_tokenizer, character_detokenizer
-            >>> OneToOnePerturbation.from_dictionary({'.', '!'},
+            >>> OneToOnePerturbation.from_dictionary({'.': '!'},
             >>>                                      label_from='not_excited',
             >>>                                      label_to='excited',
             >>>                                      tokenizer=character_tokenizer,
