@@ -1,14 +1,12 @@
 """Sensitivity testing, for fairness and robustness."""
 
-from typing import Iterator, List, Optional, Tuple, Union
+from typing import List, Optional, Tuple, Union
 
-import numpy as np
 from genbase import add_callargs
-from instancelib.analysis.base import (BinaryModelMetrics,
-                                       MulticlassModelMetrics, label_metrics)
+from instancelib.analysis.base import label_metrics
 from instancelib.environment.text import TextEnvironment
 from instancelib.instances.base import InstanceProvider
-from instancelib.instances.text import MemoryTextInstance, TextInstanceProvider
+from instancelib.instances.text import TextInstanceProvider
 from instancelib.labels.memory import MemoryLabelProvider
 from instancelib.machinelearning.base import AbstractClassifier
 from instancelib.typehints import LT
@@ -17,7 +15,7 @@ from text_sensitivity.data.generate import from_pattern
 from text_sensitivity.data.random.string import (RandomString,
                                                  combine_generators)
 from text_sensitivity.perturbation.base import Perturbation
-from text_sensitivity.return_types import SuccessTest
+from text_sensitivity.return_types import LabelMetrics, MeanScore, SuccessTest
 
 
 def apply_perturbation(dataset: Union[InstanceProvider, TextEnvironment],
@@ -57,7 +55,7 @@ def apply_perturbation(dataset: Union[InstanceProvider, TextEnvironment],
             attributes.extend(labels) if isinstance(labels, list) else attributes.append(labels)
 
     instanceprovider = TextInstanceProvider(new_data)
-    instanceprovider.add_range(*dataset.dataset.get_all())
+    instanceprovider.add_range(*dataset.get_all())
     labelprovider = MemoryLabelProvider.from_tuples(attributes)
 
     return instanceprovider, labelprovider
@@ -71,10 +69,11 @@ def equal_ground_truth(ground_truth, instances):
         yield (key, ground_truth._labeldict[parent_key])
 
 
+@add_callargs
 def compare_metric(env: TextEnvironment,
                    model: AbstractClassifier,
-                   perturbation: Perturbation
-                   ) -> Iterator[Tuple[LT, LT, Union[BinaryModelMetrics, MulticlassModelMetrics]]]:
+                   perturbation: Perturbation,
+                   **kwargs) -> LabelMetrics:
     """Get metrics for each ground-truth label and attribute.
 
     Examples:
@@ -95,10 +94,12 @@ def compare_metric(env: TextEnvironment,
         model (AbstractClassifier): Black-box model to compare metrics on.
         perturbation (Perturbation): Peturbation to apply.
 
-    Yields:
-        Iterator[Sequence[Tuple[LT, LT, Union[BinaryModelMetrics, MulticlassModelMetrics]]]]: Original label (before
-            perturbation), perturbed label (after perturbation) and metrics for label-attribute pair.
+    Returns:
+        LabelMetrics: Original label (before perturbation), perturbed label (after perturbation) 
+        and metrics for label-attribute pair.
     """
+    callargs = kwargs.pop('__callargs__', None)
+
     # Apply perturbations and get attributes
     instances, attributes = apply_perturbation(env, perturbation)
 
@@ -108,13 +109,16 @@ def compare_metric(env: TextEnvironment,
     # Expectation (for now that labels should remain equal)
     ground_truth = MemoryLabelProvider.from_tuples(list(equal_ground_truth(env.labels, instances)))
 
-    for label in list(model_predictions.labelset):
-        for attribute in list(attributes.labelset):
-            metrics = label_metrics(model_predictions,
-                                    ground_truth,
-                                    attributes.get_instances_by_label(attribute),
-                                    label)
-            yield label, attribute, metrics
+    lm = [(label, attribute, label_metrics(model_predictions,
+                                           ground_truth,
+                                           attributes.get_instances_by_label(attribute),
+                                           label))
+                    for attribute in list(attributes.labelset)
+                    for label in list(model_predictions.labelset)]
+
+    return LabelMetrics(instances=instances,
+                        label_metrics=lm,
+                        callargs=callargs)  
 
 
 def compare_accuracy(*args, **kwargs):
@@ -245,10 +249,11 @@ def invariance(pattern: str,
                        callargs=callargs)
 
 
+@add_callargs
 def mean_score(pattern: str,
                model: AbstractClassifier,
                selected_label: Optional[LT] = None,
-               **kwargs) -> Tuple[float, List[Tuple[MemoryTextInstance, float]]]:
+               **kwargs) -> MeanScore:
     """Calculate mean (probability) score for a given label, for data generated from a pattern.
 
     Args:
@@ -258,8 +263,10 @@ def mean_score(pattern: str,
             Defaults to None.
 
     Returns:
-        Tuple[float, List[Tuple[LT, float]]]: Mean score for the selected label, generated instances and label scores.
+        MeanScore: Mean score for the selected label, generated instances and label scores.
     """
+    callargs = kwargs.pop('__callargs__', None)
+
     instances, _ = from_pattern(pattern, **kwargs)
     predictions = model.predict_proba_provider(instances)
 
@@ -273,4 +280,8 @@ def mean_score(pattern: str,
     predictions_by_label = [(instances[id], [proba for label, proba in list(probas) if label == selected_label][0])
                             for id, probas in predictions]
 
-    return np.mean([p for id, p in predictions_by_label]), predictions_by_label
+    return MeanScore(scores=[p for _, p in predictions_by_label],
+                     label=selected_label,
+                     instances=instances,
+                     callargs=callargs,
+                     **kwargs)
